package com.xtremelabs.robolectric.shadows;

import android.content.Context;
import android.view.View;
import android.widget.TextView;
import android.widget.Toast;
import com.xtremelabs.robolectric.util.Implementation;
import com.xtremelabs.robolectric.util.Implements;
import com.xtremelabs.robolectric.util.RealObject;

import java.util.ArrayList;

import static com.xtremelabs.robolectric.Robolectric.shadowOf;

@SuppressWarnings({"UnusedDeclaration"})
@Implements(Toast.class)
public class ShadowToast {
    private static ArrayList<Toast> shownToasts = new ArrayList<Toast>();

    private String text;
    private int gravity;
    private View view;

    @RealObject Toast toast;

    @Implementation
    public static Toast makeText(Context context, int resId, int duration) {
        return makeText(context, context.getResources().getString(resId), duration);
    }

    @Implementation
    public static Toast makeText(Context context, CharSequence text, int duration) {
        Toast toast = new Toast(null);
        shadowOf(toast).text = text.toString();
        return toast;
    }

    @Implementation
    public void show() {
        shownToasts.add(toast);
    }

    @Implementation
    public void setView(View view) {
        this.view = view;
    }

    @Implementation
    public View getView() {
        return view;
    }

    @Implementation
    public void setGravity(int gravity, int xOffset, int yOffset) {
        this.gravity = gravity;
    }

    @Implementation
    public int getGravity() {
        return gravity;
    }

    public static void reset() {
        shownToasts.clear();
    }

    public static int shownToastCount() {
        return shownToasts.size();
    }

    public static boolean showedCustomToast(CharSequence message, int layoutResourceIdToCheckForMessage) {
        for (Toast toast : shownToasts) {
            String text = ((TextView) toast.getView().findViewById(layoutResourceIdToCheckForMessage)).getText().toString();
            if (text.equals(message.toString())) {
                return true;
            }
        }
        return false;
    }

    public static boolean showedToast(CharSequence message) {
        for (Toast toast : shownToasts) {
            String text = shadowOf(toast).text;
            if (text != null && text.equals(message.toString())) {
                return true;
            }
        }
        return false;
    }

    public static String getTextOfLatestToast() {
        return shadowOf(shownToasts.get(0)).text;
    }

    public static Toast getLatestToast() {
        return shownToasts.get(shownToasts.size() - 1);
    }
}
